/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.als;

import fr.amap.commons.util.ProcessingListener;
import fr.amap.lidar.amapvox.voxelisation.Voxelization;
import fr.amap.lidar.amapvox.voxelisation.configuration.ALSVoxCfg;
import java.io.File;
import fr.amap.lidar.amapvox.commons.VoxelSpaceInfos;
import fr.amap.lidar.amapvox.util.AVoxTask;
import javax.vecmath.Matrix4d;

/**
 *
 * @author Julien Heurtebize (julienhtbe@gmail.com)
 */
public class LasVoxelisation extends AVoxTask {

    private final ALSVoxCfg cfg;
    private PointsToShot alsShotBuilder;

    public LasVoxelisation(File file, int ncpu) {
        super(file, ncpu);
        cfg = new ALSVoxCfg();
    }

    @Override
    public String getName() {
        return "ALS voxelisation";
    }

    @Override
    public void init() throws Exception {

        cfg.readConfiguration(file);
        cfg.getVoxelParameters().infos.setType(VoxelSpaceInfos.Type.ALS);

        // vop matrix
        Matrix4d vop = cfg.getVopMatrix();
        if (null == vop) {
            vop = new Matrix4d();
            vop.setIdentity();
        }

        // ALS shot builder
        alsShotBuilder = new PointsToShot(cfg.getTrajectoryFile(), cfg.getInputFile(), vop);
        // add listener to monitor progress of the ALS shot builder
        alsShotBuilder.addProcessingListener(new ProcessingListener() {

            @Override
            public void processingStepProgress(String progressMsg, long progress, long max) {
                LasVoxelisation.this.fireProgress(progressMsg, progress, max);
            }

            @Override
            public void processingFinished(float duration) {
                LasVoxelisation.this.fireFinished(duration);
            }
        });
        // initializes the ALS shot builder
        alsShotBuilder.init();
    }

    @Override
    public File[] call() throws Exception {
        return new Voxelization(cfg).voxelization(alsShotBuilder.iterator());
    }
}
