/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.tls;

import fr.amap.lidar.format.jleica.LShot;
import fr.amap.lidar.format.jleica.ptg.PTGScan;
import fr.amap.lidar.format.jleica.LPointShotExtractor;
import fr.amap.commons.math.matrix.Mat3D;
import fr.amap.commons.math.matrix.Mat4D;
import fr.amap.commons.math.vector.Vec3D;
import fr.amap.commons.math.vector.Vec4D;
import fr.amap.commons.util.IteratorWithException;
import fr.amap.lidar.amapvox.shot.Shot;
import fr.amap.lidar.amapvox.voxelisation.configuration.TLSVoxCfg;
import java.io.File;
import java.util.Iterator;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import org.apache.log4j.Logger;

/**
 *
 * @author calcul
 */
public class PTGVoxelisation extends MultiScanTLSVoxelisation {

    public PTGVoxelisation(File file, int ncpu) {
        super(file, ncpu, SingleScanPTGVoxelisation.class);
    }

    class SingleScanPTGVoxelisation extends TLSVoxelisation {

        private final Logger LOGGER = Logger.getLogger(SingleScanPTGVoxelisation.class);

        public SingleScanPTGVoxelisation(File file, int ncpu) throws Exception {
            super(file, ncpu);
        }
        
        public SingleScanPTGVoxelisation(TLSVoxCfg cfg, int ncpu) throws Exception {
            super(cfg, ncpu);
        }

        @Override
        public String getName() {
            return "TLS voxelisation (PTG)";
        }

        @Override
        public File[] call() throws Exception {

            LOGGER.info("PTG extraction is started");

            PTGScan pTGScan = new PTGScan();
            pTGScan.openScanFile(inputFile);

            Iterator<LShot> iterator = new LPointShotExtractor(pTGScan).iterator();

            IteratorWithException<Shot> it = new IteratorWithException() {

                private int index = 0;

                @Override
                public boolean hasNext() throws Exception {
                    return iterator.hasNext();
                }

                @Override
                public Object next() throws Exception {
                    LShot shot = iterator.next();
                    if (shot != null) {
                        Vec4D locVector = Mat4D.multiply(transfMatrix, new Vec4D(shot.origin.x, shot.origin.y, shot.origin.z, 1.0d));
                        Vec3D uVector = Mat3D.multiply(rotation, new Vec3D(shot.direction.x, shot.direction.y, shot.direction.z));
                        return new Shot(index++, new Point3d(locVector.x, locVector.y, locVector.z), new Vector3d(uVector.x, uVector.y, uVector.z), shot.ranges);
                    } else {
                        return null;
                    }
                }
            };

            File[] files = voxelAnalysis.voxelization(it);

            fireSucceeded();
            
            return files;

        }

    }
}
