/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.tls;

import fr.amap.commons.math.matrix.Mat3D;
import fr.amap.commons.math.matrix.Mat4D;
import fr.amap.commons.math.vector.Vec3D;
import fr.amap.commons.math.vector.Vec4D;
import fr.amap.amapvox.io.tls.rxp.RxpExtraction;
import fr.amap.commons.util.IteratorWithException;
import fr.amap.lidar.amapvox.voxelisation.configuration.TLSVoxCfg;
import java.io.File;
import java.util.Iterator;
import java.util.logging.Level;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;
import org.apache.log4j.Logger;

/**
 *
 * @author Julien Heurtebize (julienhtbe@gmail.com)
 */
public class RxpVoxelisation extends TLSVoxelisation {

    private final static Logger LOGGER = Logger.getLogger(RxpVoxelisation.class);

    public RxpVoxelisation(File file, int ncpu) {
        super(file, ncpu);
    }
    
    public RxpVoxelisation(TLSVoxCfg cfg, int ncpu) {
        super(cfg, ncpu);
    }
    
    @Override
    public String getName() {
        return "TLS voxelisation (RXP)";
    }

    @Override
    public File[] call() throws Exception {

        //System.out.println(Thread.currentThread().getName());
        LOGGER.info("RXP extraction is started");

        RxpExtraction rxpExtraction = new RxpExtraction();
        int result = rxpExtraction.openRxpFile(inputFile, RxpExtraction.REFLECTANCE, RxpExtraction.DEVIATION);

        if (result != 0) {
            LOGGER.error("Extraction aborted");
            return null;
        }

        Iterator<fr.amap.amapvox.io.tls.rxp.Shot> iterator = cfg.isEnableEmptyShotsFiltering()
                ? new FalseEmptyShotRemover(rxpExtraction.iterator()).iterator()
                : rxpExtraction.iterator();

        IteratorWithException<fr.amap.lidar.amapvox.shot.Shot> it = new IteratorWithException() {

            @Override
            public boolean hasNext() {
                return iterator.hasNext();
            }

            @Override
            public fr.amap.lidar.amapvox.shot.Shot next() throws Exception {
                fr.amap.amapvox.io.tls.rxp.Shot shot = iterator.next();
                Vec4D locVector = Mat4D.multiply(transfMatrix, new Vec4D(shot.origin.x, shot.origin.y, shot.origin.z, 1.0d));
                Vec3D uVector = Mat3D.multiply(rotation, new Vec3D(shot.direction.x, shot.direction.y, shot.direction.z));
                return new fr.amap.lidar.amapvox.shot.Shot(rxpExtraction.getShotID(), new Point3d(locVector.x, locVector.y, locVector.z), new Vector3d(uVector.x, uVector.y, uVector.z), shot.ranges);
            }
        };

        File[] files = voxelAnalysis.voxelization(it);

        // close rxp reader
        rxpExtraction.close();
        
        fireSucceeded();

        return files;
    }

    public static void main(String... args) {
        try {
            // voxelisation
            RxpVoxelisation voxelisation = new RxpVoxelisation(new File("/home/pverley/Downloads/cfg.xml"), 1);
            voxelisation.init();
            voxelisation.call();
        } catch (Exception ex) {
            java.util.logging.Logger.getLogger(RxpVoxelisation.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
