/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization;

import com.jogamp.opengl.GLProfile;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.VoxelSpaceFile;
import fr.amap.lidar.amapvox.visualization.views.VisualizationFrame;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.Space;
import java.io.InputStream;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.UnsupportedLookAndFeelException;

/**
 * Main class of the Voxelization project.
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class Visualization {

    /**
     * Main function of the project (only used to testing)
     *
     * @param args Arguments (not needed)
     */
    public static void main(String[] args) {

        Visualization.launch(Visualization.class.getResourceAsStream("/data/test.vox"));
    }

    /**
     * Function used to launch a Visualization application.
     *
     * @param in Voxel space file to visualize.
     */
    public static void launch(final InputStream in) {
        System.setProperty("jogl.disable.openglcore", "true");
        /* Check if OpenGL 3.0 is available */
        if (!GLProfile.isAvailable(GLProfile.GL3bc)) {
            System.out.println("OpenGL version 3 not available...");
            /* Display a window if OpenGL not available */
            JOptionPane.showMessageDialog(new JFrame(),
                    "OpenGL version 3 is not available.",
                    "AMAPVox",
                    JOptionPane.ERROR_MESSAGE);
            System.exit(0);
        } else {
            /* Continue to launch the application */
            System.out.println("OpenGL version 3 available");

            /* Set the Nimbus look and feel */
            try {
                javax.swing.UIManager.
                        setLookAndFeel("javax.swing.plaf.nimbus.NimbusLookAndFeel");
            } catch (ClassNotFoundException | InstantiationException | IllegalAccessException | UnsupportedLookAndFeelException ex) {
                // fallback L&F.
            }

            /* Parse the file */
            final VoxelSpaceFile file = VoxelSpaceFile.parse(in);

            /* Check if the file has been parsed */
            if (file != null) {
                /* Create and display the form */
                java.awt.EventQueue.invokeLater(() -> {
                    VisualizationFrame frame = new VisualizationFrame();
                    frame.setVisible(true);

                    frame.getView3DInternalFrame().getPanelVisualization().setSpace(
                            new Space(file));

                    frame.getPropertiesInternalFrame().setPropertyNames(file.getNames());
                });
            } else {
                JOptionPane.showMessageDialog(new JFrame(),
                        "Vox file cannot be read",
                        "AMAPVox",
                        JOptionPane.ERROR_MESSAGE);
                System.exit(0);
            }
        }
    }

}
