/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.controllers.voxels;

/**
 * Simple class to create visualization filter object.
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class Filter {
    
    /**
     * The min value to display.
     */
    private final float min;
    
    /**
     * The max value to display.
     */
    private final float max;
    
    /**
     * If this filter will hide the NaN values.
     */
    private boolean hideNaN;

    /**
     * Create a filter with the min and max values.
     * @param min The min value to display.
     * @param max The max value to display.
     * @param hideNaN If this filter will hide the NaN values.
     */
    public Filter(final float min, final float max, final boolean hideNaN) {
        this.min = min;
        this.max = max;
        this.hideNaN = hideNaN;
    }

    /**
     * Get the max value to display.
     * @return 
     */
    public final float getMin() {
        return min;
    }

    /**
     * Get the max value to display.
     * @return 
     */
    public final float getMax() {
        return max;
    }

    /**
     * Get if this filter will hide the NaN values.
     * @return 
     */
    public boolean isHideNaN() {
        return hideNaN;
    }
    
    /**
     * Check if the specified value is between the min and max value.
     * @param value
     * @return False if between or False.
     */
    public final boolean isHiden(final float value) {
        if (max >= min) {
            if (value > max || value < min) {
                return true;
            }
        } else {
            if (value >= max && value <= min) {
                return true;
            }
        }
        
        return hideNaN && Float.isNaN(value);
    }
    
}
