/*
 * This software is governed by the CeCILL-B license under French law and
 * abiding by the rules of distribution of free software. You can use, modify
 * and/ or redistribute the software under the terms of the CeCILL-B license as
 * circulated by CEA, CNRS and INRIA at the following URL "http://www.cecill.info".
 * 
 * As a counterpart to the access to the source code and rights to copy, modify
 * and redistribute granted by the license, users are provided only with a
 * limited warranty and the software's author, the holder of the economic
 * rights, and the successive licensors have only limited liability.
 * 
 * In this respect, the user's attention is drawn to the risks associated with
 * loading, using, modifying and/or developing or reproducing the software by
 * the user in light of its specific status of free software, that may mean that
 * it is complicated to manipulate, and that also therefore means that it is
 * reserved for developers and experienced professionals having in-depth
 * computer knowledge. Users are therefore encouraged to load and test the
 * software's suitability as regards their requirements in conditions enabling
 * the security of their systems and/or data to be ensured and, more generally,
 * to use and operate it in the same conditions as regards security.
 * 
 * The fact that you are presently reading this means that you have had
 * knowledge of the CeCILL-B license and that you accept its terms.
 */
package fr.amap.lidar.amapvox.visualization.views;

import fr.amap.lidar.amapvox.visualization.controllers.voxels.VoxelSpaceFile;
import fr.amap.lidar.amapvox.visualization.controllers.voxels.Space;
import java.awt.BorderLayout;

/**
 *
 * @author Tristan Muller (tristan.muller@cirad.fr)
 */
public class VisualizationFrame extends javax.swing.JFrame {

    /** Creates new form MainFrame */
    public VisualizationFrame() {
        initComponents();
        initChildren();
        setLocationRelativeTo(getParent());
        setIconImage(new javax.swing.ImageIcon(getClass().
                getResource("/icons/visualization.png")).getImage());
    }

    @Override
    public void dispose() {
        super.dispose();
        view3DInternalFrame.getPanelVisualization().getGLWindow().destroy();
    }

    /** This method is called from within the constructor to initialize the
     * form. WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    @SuppressWarnings("unchecked")
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("AMAPVox - Visualization");
        setMinimumSize(new java.awt.Dimension(640, 480));
        setPreferredSize(new java.awt.Dimension(900, 600));

        pack();
    }// </editor-fold>//GEN-END:initComponents

    /**
     * This method is called after the components initialization to add their
     * children.
     */
    private void initChildren() {
        propertiesInternalFrame = new PropertiesInternalFrame(this);
        add(propertiesInternalFrame, BorderLayout.WEST);

        view3DInternalFrame = new View3DInternalFrame();
        add(view3DInternalFrame, BorderLayout.CENTER);
    }

    /**
     * Change the current voxel file.
     * @param voxelSpaceFile 
     */
    public void setVoxelSpaceFile(VoxelSpaceFile voxelSpaceFile) {
        this.voxelSpaceFile = voxelSpaceFile;
        propertiesInternalFrame.setPropertyNames(voxelSpaceFile.getNames());
    }

    /**
     * Get the current voxel file.
     * @return 
     */
    public final VoxelSpaceFile getVoxelSpaceFile() {
        return voxelSpaceFile;
    }
    
    /**
     * Get the view 3D internal frame.
     * @return 
     */
    public final View3DInternalFrame getView3DInternalFrame() {
        return view3DInternalFrame;
    }

    /**
     * Get the current properties internal frame.
     * @return 
     */
    public final PropertiesInternalFrame getPropertiesInternalFrame() {
        return propertiesInternalFrame;
    }
    
    /**
     * Get the current 3D voxel space object.
     * @return 
     */
    public final Space getSpace() {
        return view3DInternalFrame.getPanelVisualization().getSpace();
    }

    private VoxelSpaceFile voxelSpaceFile;
    
    // Children declaration
    private PropertiesInternalFrame propertiesInternalFrame;
    private View3DInternalFrame view3DInternalFrame;
    // End of children declaration

    // Variables declaration - do not modify//GEN-BEGIN:variables
    // End of variables declaration//GEN-END:variables
}
